const path = require('path')
const metadata = require('read-metadata')
const exists = require('fs').existsSync
const getGitUser = require('./git-user')
const validateName = require('validate-npm-package-name')
const changeCase = require('change-case')

/**
 * Read prompts metadata.
 *
 * @param {String} dir
 * @param {Object} nameOpts
 * @return {Object}
 */

module.exports = function options (dir, name) {
  const opts = getMetadata(dir)

  setDefault(opts, 'name', changeValueCase(name, 'param'))
  setValidateName(opts)

  setDefault(opts, 'appName', changeValueCase(name, 'pascel'))
  setDefault(opts, 'objectName', changeValueCase(name, 'camel'))

  setDefault(opts, 'nameLower', changeValueCase(name, 'lower'))
  setDefault(opts, 'nameUpper', changeValueCase(name, 'upper'))
  setDefault(opts, 'nameConst', changeValueCase(name, 'constant'))

  const author = getGitUser()
  if (author) {
    setDefault(opts, 'author', author)
  }

  return opts
}

function changeValueCase (value, caseType) {
  if (caseType === 'camel') {
    return changeCase.camel(value)
  } else if (caseType === 'lower') {
    return changeCase.lower(value)
  } else if (caseType === 'param') {
    return changeCase.param(value)
  } else if (caseType === 'pascal') {
    return changeCase.pascal(value)
  } else if (caseType === 'title') {
    return changeCase.title(value)
  } else if (caseType === 'upper') {
    return changeCase.upper(value)
  } else if (caseType === 'constant') {
    return changeCase.constant(value)
  } else {
    return value
  }
}

/**
 * Gets the metadata from either a meta.json or meta.js file.
 *
 * @param  {String} dir
 * @return {Object}
 */

function getMetadata (dir) {
  const json = path.join(dir, 'meta.json')
  const js = path.join(dir, 'meta.js')
  let opts = {}

  if (exists(json)) {
    opts = metadata.sync(json)
  } else if (exists(js)) {
    const req = require(path.resolve(js))
    if (req !== Object(req)) {
      throw new Error('meta.js needs to expose an object')
    }
    opts = req
  }

  return opts
}

/**
 * Set the default value for a prompt question
 *
 * @param {Object} opts
 * @param {String} key
 * @param {String} val
 */

function setDefault (opts, key, val) {
  if (opts.schema) {
    opts.prompts = opts.schema
    delete opts.schema
  }
  const prompts = opts.prompts || (opts.prompts = {})
  if (!prompts[key] || typeof prompts[key] !== 'object') {
    prompts[key] = {
      'type': 'string',
      'default': val
    }
  } else {
    prompts[key]['default'] = val
  }
}

function setValidateName (opts) {
  const name = opts.prompts.name
  const customValidate = name.validate
  name.validate = function (name) {
    const its = validateName(name)
    if (!its.validForNewPackages) {
      const errors = (its.errors || []).concat(its.warnings || [])
      return 'Sorry, ' + errors.join(' and ') + '.'
    }
    if (typeof customValidate === 'function') return customValidate(name)
    return true
  }
}
